#include <dirent.h>
#include <iostream>
#include <libguile.h>
#include "generators.h"
#include "mods.h"
#include "paths.h"


SCM register_scene_generator(SCM name, SCM type, SCM proc)
{
  SceneGenerator generator {scm_to_locale_string(name), scm_to_locale_string(type), proc };
  register_generator(generator);
  printf ("Register: %s (%s)\n", generator.name.c_str(), generator.type.c_str());
  //scm_call_1(proc, scm_from_int(-1));
  return SCM_UNSPECIFIED;
}

void init_dungeon_master_module(void *unused)
{
  scm_c_define_gsubr("register-scene-generator", 3, 0, 0, (scm_t_subr)register_scene_generator);
  scm_c_export("register-scene-generator", NULL);
}

void scm_init_dungeon_master_module()
{
  scm_c_define_module("dungeon-master", init_dungeon_master_module, NULL);
}

void load_mods()
{
  scm_init_guile();
  scm_init_dungeon_master_module();

  std::set<std::string> paths = get_dm_paths();
  DIR* mods_dir;
  struct dirent* mod;
  std::string
    mods_path,
    mod_main;

  for (const std::string &path : paths) {
    mods_path = path + DIR_DELIM + "mods";
    mods_dir = opendir(mods_path.c_str());
    if (mods_dir != NULL) {
      while (mod = readdir(mods_dir)) {
	if (strcmp(mod->d_name, ".") != 0 && strcmp(mod->d_name, "..") != 0) {
	  mod_main = mods_path + DIR_DELIM + mod->d_name + DIR_DELIM + "main.scm";
	  scm_primitive_load(scm_from_locale_string(mod_main.c_str()));
	}
      }
    }
    closedir(mods_dir);
  }
}
